<?php
/*-------------------------------------------------------
*
*   LiveStreet Engine Social Networking
*   Copyright © 2008 Mzhelskiy Maxim
*
*--------------------------------------------------------
*
*   Official site: www.livestreet.ru
*   Contact e-mail: rus.engine@gmail.com
*
*   GNU General Public License, version 2:
*   http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
*
---------------------------------------------------------
*/

/**
 * Объект маппера для работы с БД
 *
 * @package modules.stream
 * @since 1.0
 */
class ModuleStream_MapperStream extends Mapper
{
    /**
     * Добавление события в БД
     *
     * @param ModuleStream_EntityEvent $oObject
     * @return int|bool
     */
    public function AddEvent($oObject)
    {
        $sql = "INSERT INTO ".Config::Get('db.table.stream_event')." SET ?a ";
        if ($iId=$this->oDb->query($sql, $oObject->_getData())) {
            return $iId;
        }
        return false;
    }
    /**
     * Получает событие по типу и его ID
     *
     * @param string $sEventType	Тип
     * @param int $iTargetId	ID владельца события
     * @param int|null $iUserId	ID пользователя
     * @return ModuleStream_EntityEvent
     */
    public function GetEventByTarget($sEventType, $iTargetId, $iUserId=null)
    {
        $sql = "SELECT * FROM ".Config::Get('db.table.stream_event')." WHERE target_id = ?d AND event_type = ? { AND user_id = ?d } ";
        if ($aRow=$this->oDb->selectRow($sql, $iTargetId, $sEventType, is_null($iUserId) ? DBSIMPLE_SKIP : $iUserId)) {
            return Engine::GetEntity('ModuleStream_EntityEvent', $aRow);
        }
        return null;
    }
    /**
     * Обновление события
     *
     * @param ModuleStream_EntityEvent $oObject	Объект события
     * @return int
     */
    public function UpdateEvent($oObject)
    {
        $sql = "UPDATE ".Config::Get('db.table.stream_event')." SET ?a WHERE id = ?d ";
        return $this->oDb->query($sql, $oObject->_getData(array('publish')), $oObject->getId());
    }
    /**
     * Получение типов событий, на которые подписан пользователь
     *
     * @param int $iUserId	ID пользователя
     * @return array
     */
    public function getTypesList($iUserId)
    {
        $sql = 'SELECT event_type FROM ' . Config::Get('db.table.stream_user_type') . ' WHERE user_id = ?d';
        $aRet = $this->oDb->selectCol($sql, $iUserId);
        return $aRet;
    }
    /**
     * Получение списка пользователей, на которых подписан пользователь
     *
     * @param int $iUserId	ID пользователя
     * @return array
     */
    public function getUserSubscribes($iUserId)
    {
        $sql = 'SELECT target_user_id FROM ' . Config::Get('db.table.stream_subscribe') . ' WHERE user_id = ?d';
        return $this->oDb->selectCol($sql, $iUserId);
    }
    /**
     * Чтение событий
     *
     * @param array $aEventTypes	Список типов событий
     * @param array|null $aUsersList	Список пользователей, чьи события читать
     * @param int $iCount	Количество
     * @param int $iFromId	ID события с которого начинать выборку
     * @param int $current_user_id ID текущего пользователя (0 для неавторизованного)
     * @param bool $no_privacy_restrictions true для администратора
     * @return array
     */
    public function Read($aEventTypes, $aUsersList, $iCount, $iFromId, int $current_user_id = 0, bool $no_privacy_restrictions = false)
    {
        $rows = [];
        if($no_privacy_restrictions) {
            $sql = 'SELECT * FROM '.Config::Get('db.table.stream_event').'
    WHERE
        publish = 1
        and event_type IN (?a) 
        { AND user_id IN (?a) }
        { AND id < ?d }	
    ORDER BY id DESC
    { LIMIT 0,?d }';
            $rows = $this->oDb->select($sql, $aEventTypes, (!is_null($aUsersList) && count($aUsersList)) ? $aUsersList : DBSIMPLE_SKIP, !is_null($iFromId) ? $iFromId : DBSIMPLE_SKIP, !is_null($iCount) ? $iCount : DBSIMPLE_SKIP);
        }
        else {
            $sql = $this->buildReadEventsQuery($aEventTypes ?? [], $aUsersList ?? [], $iCount, $iFromId, $current_user_id, false);
            $rows = $this->oDb->select($sql);
        }
        $aReturn = [];
        if(!empty($rows)) {
            foreach($rows as $row) {
                $aReturn[]=Engine::GetEntity('Stream_Event', $row);
            }
        }
        return $aReturn;
    }
    /**
     * Количество событий для пользователя
     * @deprecated Запрос вызывает высокую нагрузку и выполняется очень долго. Используйте механизм бесконечной ленты для загрузки событий.
     * @param array $aEventTypes	Список типов событий
     * @param int|array|null $aUserId	ID пользователя
     * @param int $current_user_id ID текущего пользователя (0 для неавторизованного)
     * @param bool $no_privacy_restrictions true для администратора
     * @return int
     */
    public function GetCount($aEventTypes, $aUsersList, int $current_user_id = 0, bool $no_privacy_restrictions = false)
    {
        if(!is_null($aUsersList) && !is_array($aUsersList))
            $aUsersList = [$aUsersList];

        $aDelaultEventTypes = [
            //'add_wall', todo: не используется
            'add_topic',
            'add_comment',
            'add_blog',
            'vote_topic',
            'vote_comment',
            'vote_blog',
            'vote_user',
            'add_friend',
            'join_blog'
        ];
        $not_skipped = count(array_diff_key($aDelaultEventTypes, $aEventTypes));

        $row = [];
        if($no_privacy_restrictions) {
            $sql = 'SELECT count(*) as c FROM '.Config::Get('db.table.stream_event').'
    WHERE
        publish = 1
        and event_type IN (?a) 
        { AND user_id IN (?a) }';
            $row = $this->oDb->selectRow($sql, ($not_skipped) ? $aEventTypes : DBSIMPLE_SKIP, (!is_null($aUsersList) and count($aUsersList)) ? $aUsersList : DBSIMPLE_SKIP);
        }
        else {
            $sql = $this->buildReadEventsQuery($not_skipped ? $aEventTypes : [], $aUsersList ?? [], null, null, $current_user_id, true);
            $row = $this->oDb->selectRow($sql);
        }
        if(!empty($row))
            return $row['c'];
        return 0;
    }

    private function buildReadEventsQuery(array $eventTypes, array $usersList, ?int $count = null, ?int $startFromId = null, int $current_user_id = 0, bool $count_only = false): string
    {
        $event_types = []; $queries = [];
        $br = Engine::getInstance()->GetModuleObject('Stream')->getEventTargetToBlogRelation();
        $ec = Engine::getInstance()->GetModuleObject('Stream')->getEventTargetPublishingTerms();
        $valid_friend_status_sql = ModuleUser::USER_FRIEND_OFFER.','.ModuleUser::USER_FRIEND_ACCEPT;
        // проходим по списку типов событий
        foreach(Engine::getInstance()->GetModuleObject('Stream')->getEventTypes() as $eventCode=>$eventType) {
            // задана фильтрация по типам событий и этот тип не запрашивается, пропускаем
            if(!empty($eventTypes) && !in_array($eventCode,$eventTypes)) continue;
            if(!isset($event_types[$eventType['related']])) {
                // такой объект события ещё не добавляли, начинаем собирать подзапрос
                $event_types[$eventType['related']] = [];
                $tblalias = 'lss_'.substr($eventType['related'],0,1);
                $queries[$eventType['related']] = 'select '.($count_only ? 'count('.$tblalias.'.id) as c' : $tblalias.'.*').' from ls_stream_event '.$tblalias.' use index(PRIMARY)
    left join ls_user_privacy lsup using(user_id)';
                // если нужно проверять закрытость блога, к которому относится объект события, собираем цепочку джойнов от таблицы событий до таблицы блогов.
                if(isset($br[$eventType['related']])) {
                    $prev_field = $tblalias.'.target_id';
                    foreach($br[$eventType['related']] as $br_table=>$br_field) {
                        $queries[$eventType['related']] .= '
    left join '.$br_table.' on '.$br_table.'.'.$br_field[0].' = '.$prev_field;
                        $prev_field = $br_table.'.'.$br_field[1];
                    }
                    $queries[$eventType['related']] .= '
    left join ls_blog lsb on '.$prev_field.' = lsb.blog_id';
                    // если запрос от авторизованного пользователя, так же джойним таблицу участников блога
                    if(!empty($current_user_id)) {
                        $queries[$eventType['related']] .= '
    left join '.Config::Get('db.table.blog_user').' lsbu on lsbu.blog_id = lsb.blog_id and lsbu.user_id = '.$current_user_id. ' and lsbu.user_role >= 1';
                    }
                }
                // иначе если нужно проверять какие-то поля объекта события, то приджойним только его таблицу. Ключевое поле должно называться кодобъекта_id
                else if(!empty($ec[$eventType['related']])) {
                    $queries[$eventType['related']] .= '
    left join '.Config::Get('db.table.'.$eventType['related']).' on '.Config::Get('db.table.'.$eventType['related']).'.'.$eventType['related'].'_id = '.$tblalias.'.target_id';
                }
                // если запрос от авторизованного пользователя, чекаем доп.уровни приватности
                if(!empty($current_user_id)) {
                    $queries[$eventType['related']] .= '
    left join '.Config::Get('db.table.friend').' lsf1 on lsf1.user_from = '.$tblalias.'.user_id and lsf1.user_to = '.$current_user_id.'
        and lsf1.status_from in('.$valid_friend_status_sql.') and lsf1.status_to in('.$valid_friend_status_sql.')
    left join '.Config::Get('db.table.friend').' lsf2 on lsf2.user_to = '.$tblalias.'.user_id and lsf2.user_from = '.$current_user_id.'
        and lsf2.status_from in('.$valid_friend_status_sql.') and lsf2.status_to in('.$valid_friend_status_sql.')';
                }
                // перечисляем условия выборки — событие опубликовано и его тип соответствует списку (который мы собираем в этом же цикле, поэтому подставим его потом)
                $queries[$eventType['related']] .= '
    where
        '.$tblalias.'.publish = 1 and '.$tblalias.'.event_type in(#EVENT_TYPES#)';
                // объект события каким-то образом относится к блогу — проверяем, что блог открытый
                if(isset($br[$eventType['related']])) {
                    $queries[$eventType['related']] .= ' and (lsb.blog_type in("open","personal")';
                    // ...или закрытый, но у пользователя есть доступ
                    if(!empty($current_user_id))
                        $queries[$eventType['related']] .= ' or lsbu.user_id is not null or lsb.user_owner_id = '.$current_user_id;
                    $queries[$eventType['related']] .= ')';
                }
                // проверяем какие-то поля объекта события, если нужно
                if(!empty($ec[$eventType['related']])) {
                    foreach($ec[$eventType['related']] as $ec_field=>$ec_value) {
                        $queries[$eventType['related']] .= ' and '.Config::Get('db.table.'.$eventType['related']).'.'.$ec_field.'='.$ec_value;
                    }
                }
                // проверяем всё, что связано с приватностью
                if(!empty($current_user_id)) {
                    $queries[$eventType['related']] .= ' and ('.$tblalias.'.user_id = '.$current_user_id.'
            or (
                (lsup.privacy_profile is null or lsup.privacy_profile < 2
                    or (lsup.privacy_profile = 2 and (lsf1.user_to is not null or lsf2.user_from is not null)))
                and (lsup.privacy_activities is null or lsup.privacy_activities < 2
                    or (lsup.privacy_activities = 2 and (lsf1.user_to is not null or lsf2.user_from is not null)))
            )
        )';
                }
                else {
                    $queries[$eventType['related']] .= ' and ifnull(lsup.privacy_profile,0) = 0 and ifnull(lsup.privacy_activities,0) = 0';
                }
                // выборка событий только от определённых пользователей
                if(count($usersList))
                    $queries[$eventType['related']] .= ' and '.$tblalias.'.user_id in('.implode(',',array_map(function($a) { return intval($a); },$usersList)).')';
                // отсечка выборки с заданного по идентификатору события
                if(!is_null($startFromId))
                    $queries[$eventType['related']] .= ' and '.$tblalias.'.id < '.$startFromId;
                // сортируем по убыванию (старше — позже/ниже) и ставим лимит, чтобы не возвращать всё. После union'а тоже будет лимит
                $queries[$eventType['related']] .= '
    order by '.$tblalias.'.id desc'.(!is_null($count) ? ' limit 0,'. $count : '');
            }
            $event_types[$eventType['related']][] = $eventCode;
        }
        $sql = 'select '.($count_only ? 'count(tmp.id) as c' : 'tmp.*').' from ('; $query_first = true;
        foreach($queries as $code=>$query) {
            $sql .= ($query_first ? '':"\n".'union ').'('.str_replace('#EVENT_TYPES#','"'.implode('","',$event_types[$code]).'"',$query).')';
            $query_first = false;
        }
        $sql .= ') tmp
order by tmp.id desc'.(!is_null($count) ? ' limit 0,'. $count : '');
        return $sql;
    }

    /**
     * Редактирование списка событий, на которые подписан юзер
     *
     * @param int $iUserId	ID пользователя
     * @param string $sEventType	Тип
     * @return bool
     */
    public function switchUserEventType($iUserId, $sEventType)
    {
        $sql = 'SELECT * FROM ' . Config::Get('db.table.stream_user_type') . ' WHERE user_id = ?d AND event_type = ?';
        if ($this->oDb->select($sql, $iUserId, $sEventType)) {
            $sql = 'DELETE FROM ' . Config::Get('db.table.stream_user_type') . ' WHERE user_id = ?d AND event_type = ?';
        } else {
            $sql = 'INSERT INTO  '. Config::Get('db.table.stream_user_type') . ' SET user_id = ?d , event_type = ?';
        }
        $this->oDb->query($sql, $iUserId, $sEventType);
    }
    /**
     * Подписать пользователя
     *
     * @param int $iUserId	ID пользователя
     * @param int $iTargetUserId	ID пользователя на которого подписываем
     */
    public function subscribeUser($iUserId, $iTargetUserId)
    {
        $sql = 'SELECT * FROM ' . Config::Get('db.table.stream_subscribe') . ' WHERE user_id = ?d AND target_user_id = ?d';
        if (!$this->oDb->select($sql, $iUserId, $iTargetUserId)) {
            $sql = 'INSERT INTO ' . Config::Get('db.table.stream_subscribe') . ' SET user_id = ?d, target_user_id = ?d';
            $this->oDb->query($sql, $iUserId, $iTargetUserId);
        }
    }
    /**
     * Отписать пользователя
     *
     * @param int $iUserId	ID пользователя
     * @param int $iTargetUserId	ID пользователя на которого подписываем
     */
    public function unsubscribeUser($iUserId, $iTargetUserId)
    {
        $sql = 'DELETE FROM ' . Config::Get('db.table.stream_subscribe') . ' WHERE user_id = ?d AND target_user_id = ?d';
        $this->oDb->query($sql, $iUserId, $iTargetUserId);
    }
    /**
     * Проверяет подписан ли пользователь на конкретного пользователя
     *
     * @param $iUserId	ID пользователя
     * @param $iTargetUserId	ID пользователя на которого подписан
     * @return bool
     */
    public function IsSubscribe($iUserId, $iTargetUserId)
    {
        $sql = 'SELECT * FROM ' . Config::Get('db.table.stream_subscribe') . ' WHERE user_id = ?d AND target_user_id = ?d LIMIT 0,1';
        if ($this->oDb->selectRow($sql, $iUserId, $iTargetUserId)) {
            return true;
        }
        return false;
    }
}
